@@ -286,8 +286,22 @@ int main(int argc, char** argv) {
   srand(getpid());
   {
     int opt;
-    while ((opt = getopt(argc,argv,"k:o:i:Vsu:")) != -1) {
+    while ((opt = getopt(argc,argv,"A:k:o:i:Vsu:")) != -1) {
       switch (opt) {
+      case 'A':
+	{ /* Scan string as hostname=username:password */
+	  char* p = strdup(optarg);
+	  char* q = strchr(p, '=');
+	  char* r = q ? strchr(q, ':') : NULL;
+	  if (!q || !r) {
+	    fprintf(stderr, "-A takes hostname=username:password\n");
+	    exit(1);
+	  } else {
+	    *q++ = *r++ = 0;
+	    add_auth(p, q, r);
+	  }
+	}
+	break;
       case 'k':
 	free(zfname); zfname = strdup(optarg);
 	break;
@@ -362,17 +376,20 @@ int main(int argc, char** argv) {
   {
     int r;
 
-    printf("verifying download...");
+    if (!no_progress)
+      printf("verifying download...");
     r = zsync_complete(zs);
     switch (r) {
     case -1:
       fprintf(stderr,"Aborting, download available in %s\n",temp_file);
       exit(2);
     case 0:
-      printf("no recognised checksum found\n");
+      if (!no_progress)
+        printf("no recognised checksum found\n");
       break;
     case 1:
-      printf("checksum matches OK\n");
+      if (!no_progress)
+        printf("checksum matches OK\n");
       break;
     }
   }
@@ -407,7 +424,8 @@ int main(int argc, char** argv) {
     printf("No filename specified for download - completed download left in %s\n",temp_file);
   }
 
-  fprintf(stderr,"used %lld local, fetched %lld\n", local_used, http_down);
+  if (!no_progress)
+    printf("used %lld local, fetched %lld\n", local_used, http_down);
   free(referer);
   free(temp_file);
   return 0;
